/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *     																	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "UCQDetailsBar.h"

/**
 * \brief Sets up the QLineEdit.
 *
 * \param parent Pointer to the parent QWidget.
 * \param utf8 If true, the bar will display UTF-8 codepoints.
 */
UCQDetailsBar::UCQDetailsBar ( QWidget* parent, bool utf8 ) :
    QLineEdit ( parent )
{
    setReadOnly ( true );
    setCursor ( Qt::IBeamCursor );
    tableModel_ = 0;
    utf8_ = utf8;
}

/**
 * \brief Slot to change the contents to the linestring of the selected item or
 * 		  the range of codepoint, if multiple items are selected.
 *
 * \param list List of indicies of the items of which the info is displayed.
 *
 * \see Letter::getCodepointAsUtf8()
 * \see Letter::getLinestringUtf8()
 * \see TableModel::getVisibleLetter()
 *
 */
void UCQDetailsBar::changeStatus ( QModelIndexList list )
{
    QString text ( "" );
    if ( !list.isEmpty () && tableModel_ != 0 ) {
        Letter* first = tableModel_->getVisibleLetter ( list.first () );
        if ( list.size () > 1 ) {
            // find the selected letter with the lowest and highest codepoint
            // values
            Letter* second = tableModel_->getVisibleLetter ( list.last() );

            uint highest = 0;
            uint lowest = 0xFFFFFFFF;

            Letter *tmp;
            for(int i = 0; i < list.size(); ++i){
                tmp = tableModel_->getVisibleLetter (list.at(i));
                if(!tmp->getCodepoint().isEmpty()){
                    if(tmp->getCodepoint().first() >= highest){
                        second  = tmp;
                        highest = tmp->getCodepoint().first();
                    }
                    if(tmp->getCodepoint().first() <= lowest){
                        first = tmp;
                        lowest = tmp->getCodepoint().first();
                    }
                }
            }

            // Show the range in the details bar
            // smallest codepoint - highest codepoint
            if ( !first->getCodepoint ().isEmpty () ) {
                if ( utf8_ ) {
                    text.append ( first->getCodepointAsUtf8 () );
                } else {
                    text.append ( UCTools::formatCodepoint (
                                      first->getCodepoint ().at ( 0 ) ) );
                }
            }
            text.append ( " - " );
            if ( !second->getCodepoint ().isEmpty () ) {
                if ( utf8_ ) {
                    text.append ( second->getCodepointAsUtf8 () );
                } else {
                    text.append ( UCTools::formatCodepoint (
                                      second->getCodepoint ().at ( 0 ) ) );
                }
            }
        } else {
            if ( utf8_ ) {
                text.append ( first->getLinestringUtf8 () );
            } else {
                text.append ( first->getLinestring () );
            }
        }
    }
    setText ( text );
    setCursorPosition ( 0 );
}

/**
 * \brief Setter for the TableModel instance, that contains the letters.
 *
 * \param tableModel TableModel instance, that contains the letters.
 */
void UCQDetailsBar::setTableModel ( TableModel* tableModel )
{
    tableModel_ = tableModel;
}

/**
 * \brief Sets UTF-8 mode
 *
 * \param utf8 If set to true, the bar will display UTF-8 codepoints.
 */
void UCQDetailsBar::setUtf8(bool utf8){
    utf8_ = utf8;
}
