/***************************************************************************
 *   Copyright (C) 2010 by Martin Burgemeister, Mathis Lindemann           *
 *     																	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "UCQTableView.h"

/**
 * \brief Sets up the QTableView without headers and horizontal scroll bar.
 *
 * \param parent Pointer to the parent QWidget.
 */
UCQTableView::UCQTableView ( QWidget* parent ) :
    QTableView ( parent )
{
    horizontalHeader ()->hide ();
    verticalHeader ()->hide ();

    setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
    // mouse tracking for the UCQDetails-window
    setMouseTracking ( true );
    // force paint event when mouse enters or leaves the widget
    viewport ()->setAttribute ( Qt::WA_Hover, true );

    fontsize_ = 22;
    QFont f = QApplication::font ();
    cellSize_ = f.pointSize () * 2;
    zoomFactor_ = 1;

    changeFont ( f );
    connect ( this, SIGNAL ( doubleClicked ( QModelIndex ) ), this,
              SLOT ( doubleClicked ( QModelIndex ) ));
}

/**
 * \brief Change the font of the letters in the view.
 *
 * The old fontsize is applied to the font. If necessary, a resizeEvent will be
 * triggered.
 * To change the fontsize use changeFontSize().
 *
 * \param font The font.
 */
void UCQTableView::changeFont ( const QFont& font )
{
    QFont temp ( font );
    temp.setPointSize ( fontsize_ );
    setFont ( temp );
    int cellSizeTest = (int) fontsize_ * 2 * zoomFactor_;
    if ( cellSizeTest != cellSize_ )
    {
        cellSize_ = cellSizeTest;
        resizeEvent ( 0 );
    }
}

/**
 * \brief Change the font size of the font for the letters.
 *
 * Changing the fontsize will always lead to a resizeEvent.
 *
 * \param size Font size.
 */
void UCQTableView::changeFontSize ( const QString& size )
{
    bool test = false;
    int s = size.toInt ( &test, 10 );
    if ( test && s > 0 )
    {
        QFont f = font ();
        fontsize_ = s;
        f.setPointSize ( s );
        changeFont ( f );
    }
}

/**
 * \brief Clears the selection in the view.
 */
void UCQTableView::clearSelections ()
{
    clearSelection ();
}

/**
 * \brief Emit the doubleClicked signal to print the letter on the console.
 *
 * \param index Index of the clicked letter.
 */
void UCQTableView::doubleClicked ( QModelIndex index )
{
    emit doubleClicked ( ( (TableModel*) model () )->getLetter ( index ) );
}

/**
 * \brief Returns a list with the indices of the selected items.
 *
 * \return List with the indices of the selected items.
 */
QModelIndexList UCQTableView::getSelectedIndexes ()
{
    return selectedIndexes ();
}

/**
 * \brief Handles keyboard events.
 *
 * Currently only Ctrl+P and Ctrl+C will be handled. Ctrl+A is ignored, because
 * it causes bugs with tables containing empty cells in the last row.
 * All other events are passed to QTableView.
 *
 * \param e The key press event.
 */
void UCQTableView::keyPressEvent ( QKeyEvent* e )
{
    switch ( e->modifiers () )
    {
        case Qt::ControlModifier:
            switch ( e->key () )
            {
                // print linestrings of selected letters in console
                case Qt::Key_P:
                    emit printSelection (
                                          ( (TableModel*) model () )->getLetters (
                                                                                   selectedIndexes () ) );
                    break;

                    // usually select all...
                case Qt::Key_A:
                    e->ignore ();
                    return;

                    // copy selected letters to clipboard
                case Qt::Key_C:
                    QList<Letter*>
                            selLetters =
                                    ( (TableModel*) model () )-> getLetters (
                                                                              selectedIndexes () );
                    QString text = "";
                    for ( int i = 0; i < selLetters.size (); ++i )
                    {
                        text.append ( selLetters.at ( i )->text () );
                    }
                    QApplication::clipboard ()->setText ( text,
                                                          QClipboard::Clipboard );
                    break;
            }
    }
    QTableView::keyPressEvent ( e );
}

/**
 * \brief Moves the cursor to the next (previous) row if right (left) is
 *        pressed in the last (first) column.
 *
 * \param cursorAction The move event.
 * \param modifiers The used modifiers.
 * \return The index the cursor is moved to.
 * \see QTableView::moveCursor ()
 */
QModelIndex UCQTableView::moveCursor ( CursorAction cursorAction,
                                       Qt::KeyboardModifiers modifiers )
{
    // move to next row if cursor in last column
    if ( cursorAction == QAbstractItemView::MoveRight
            && currentIndex ().column () == model ()->columnCount () - 1 )
    {
        return QTableView::moveCursor (QAbstractItemView::MoveNext, modifiers );
    }
    // move to previous row if cursor in first column
    if ( cursorAction == QAbstractItemView::MoveLeft
            && currentIndex ().column () == 0 )
    {
        return QTableView::moveCursor ( QAbstractItemView::MovePrevious, modifiers );
    }

    return QTableView::moveCursor ( cursorAction, modifiers );
}

/**
 * \brief Mouse move event handler to move the details window, if the mouse is
 *        moved while the right button is pressed.
 *
 * \param e The mouse event.
 */
void UCQTableView::mouseMoveEvent ( QMouseEvent* e )
{
    emit moveDetails ( e->globalX (), e->globalY () );
    QTableView::mouseMoveEvent ( e );
}

/**
 * \brief Mouse key handler that reacts on right clicks and shows the
 *        UCQDetails-window for the letter over which the mouse cursor is
 *        positioned.
 *
 * \param e The mouse event.
 * \see closeDetails()
 * \see showDetails()
 */
void UCQTableView::mousePressEvent ( QMouseEvent* e )
{
    if ( e->button () == Qt::RightButton )
    {
        emit showDetails ( e );
    } else
    {
        QTableView::mousePressEvent ( e );
        emit
        closeDetails ();
        return;
    }
}

/**
 * \brief Mouse release handler to hide the details window, if the right mouse
 *        button is released.
 *
 * \param e The mouse event.
 */
void UCQTableView::mouseReleaseEvent ( QMouseEvent* e )
{
    QTableView::mouseReleaseEvent ( e );
    if ( e->button () == Qt::RightButton )
    {
        emit closeDetails ();
    }
}

/**
 * \brief Resizes the cells.
 *
 * This method sets the size of the cells to the
 * fontsize * 2 * zoomfactor. If there is space left, that can not be
 * filled with complete cells, the cells widths is extended a little
 * (cellsize + left over space/column count).
 */
void UCQTableView::resizeCells ()
{
    if ( model () != 0 )
    {
        int columnCount = model ()->columnCount ();
        int rowCount = model ()->rowCount ();
        if ( columnCount > 0 )
        {
            int visibleWidth = size ().width ()
                    - verticalScrollBar ()->size ().width ();
            int unusedWidth = (int) visibleWidth - ( cellSize_ * zoomFactor_ )
                    * columnCount;
            int addWidth = unusedWidth / columnCount;
            int width = (int) cellSize_ * zoomFactor_ + addWidth;

            for ( int i = rowCount - 1; i >= 0; --i )
            {
                setRowHeight ( i, cellSize_ );
            }
            for ( int j = columnCount - 1; j >= 0; --j )
            {
                setColumnWidth ( j, width );
            }
        }
    }
}

/**
 * \brief Resize event handler.
 *
 * When a resize event occurs, the view will store its current selection,
 * recalculate the maximum number of columns, that can be shown and emits the
 * resize signal. After the resize it restores the old selection.
 *
 * \param e The resize event.
 */
void UCQTableView::resizeEvent ( QResizeEvent* e )
{
    // store selection
    QList<Letter*> selectedLetters;

    TableModel * m = dynamic_cast<TableModel*> ( model () );
    QModelIndexList idx = selectedIndexes ();
    for ( int i = 0; i < idx.size (); ++i )
    {
        selectedLetters.append ( m->getVisibleLetter ( idx.at ( i ) ) );
    }

    // resize model and view
    int nColumns = 0;
    if ( cellSize_ != 0 )
    {
        nColumns = (int) ( size ().width ()
                - verticalScrollBar ()->size ().width () ) / ( cellSize_
                * zoomFactor_ );
        nColumns = ( nColumns < 1 ) ? 1 : nColumns;
    }

    emit
    resized ( nColumns );
    resizeCells ();
    QWidget::resizeEvent ( e );

    // restore selection
    clearSelection ();
    QModelIndex scrollIndex;
    for ( int i = 0; i < selectedLetters.size (); ++i )
    {
        scrollIndex = m->index ( selectedLetters.at ( i ) );
        setLetterSelected ( scrollIndex, true );
    }

    // scroll to last selected item
    if ( scrollIndex.isValid () )
    {
        scrollTo ( scrollIndex );
    }
}

/**
 * \brief Slot for the scrollTo method of QTableView.
 *
 * \param index Index of the item to which to scroll to.
 */
void UCQTableView::scrollTo ( QModelIndex index )
{
    QTableView::scrollTo ( index );
}

/**
 * \brief Selection changed.
 *
 * Slot to emit the newSelection signal with all selected indices.
 *
 * \param selected The list of the newly selected indices.
 * \param deselected The list of recently deselected indices.
 */
void UCQTableView::selectionChanged ( const QItemSelection& selected,
                                      const QItemSelection& deselected )
{
    emit newSelection ( selectedIndexes () );
    QTableView::selectionChanged ( selected, deselected );
}

/**
 * \brief Set the font of the letters in the view bold.
 *
 * \param enable If true, bold is activated, else deactivated.
 */
void UCQTableView::setBold ( bool enable )
{
    QFont f = font ();
    f.setBold ( enable );
    setFont ( f );
    update ();
}

/**
 * \brief Set the font of the letters in the view italic.
 *
 * \param enable If true, italic is activated, else deactivated.
 */
void UCQTableView::setItalic ( bool enable )
{
    QFont f = font ();
    f.setItalic ( enable );
    setFont ( f );
    update ();
}

/**
 * \brief (De-)Selects a letter by its index in the model.
 *
 * \param index The QModelIndex of the letter to (de-)select.
 * \param select If true, the letter is selected, else it is deselected.
 */
void UCQTableView::setLetterSelected ( QModelIndex index, bool select )
{
    if ( index.isValid () )
    {
        selectionModel ()->select ( index,
                                    select ? QItemSelectionModel::Select
                                           : QItemSelectionModel::Deselect );
    }
}

/**
 * \brief (De-)Selects multiple letters by their indices in the model.
 *
 * \param indexList List with indices of letter to (de-)select.
 * \param select If true, the letters are selected, else they are deselected.
 */
void UCQTableView::setLettersSelected ( const QModelIndexList& indexList,
                                        bool select )
{
    QItemSelection selection;
    for ( int i = 0; i < indexList.size (); ++i )
    {
        if ( indexList.at ( i ).isValid () )
        {
            selection.append ( QItemSelectionRange ( indexList.at ( i ) ) );
        }
    }

    selectionModel ()->select ( selection,
                                select ? QItemSelectionModel::Select
                                       : QItemSelectionModel::Deselect );
}

/**
 * \brief Zoom in the table.
 *
 * This method increments the zoomfactor by 0.1, if more than one cell is in a
 * row and triggers a resizeEvent.
 */
void UCQTableView::zoomIn ()
{
    if ( model () != 0 && model ()->columnCount () > 1 ) //&& zoomFactor_ < 1.6 )
    {
        zoomFactor_ += 0.1;
        cellSize_ = (int) fontsize_ * 2 * zoomFactor_;
        resizeEvent ( 0 );
    }
}

/**
 * \brief Zoom out the table.
 *
 * This method decrements the zoomfactor by 0.1, if it larger than 0.8 and
 * triggers a resizeEvent.
 */
void UCQTableView::zoomOut ()
{
    if ( model () != 0 && model ()->columnCount () > 0 && zoomFactor_ > 0.8 )
    {
        zoomFactor_ -= 0.1;
        cellSize_ = (int) fontsize_ * 2 * zoomFactor_;
        resizeEvent ( 0 );
    }
}
